#setwd("/Users/sebastian/git/kiefer/stackoverflow-in-github/references-recall/java-high-popularity-3000-unrated/eval/config")
setwd("E:/Git/kiefer/stackoverflow-in-github/references-recall/java-high-popularity-3000-unrated/eval/config")

# READ DATA
mt <- read.csv(file="precision-recall-mt-15-100-5.csv", header=FALSE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
names(mt) <- c("min_tokens", "precision", "recall")
mt_ia <- read.csv(file="precision-recall-mt-15-100-5-ia.csv", header=FALSE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
names(mt_ia) <- c("min_tokens", "precision", "recall")
mt_ia_ii <- read.csv(file="precision-recall-mt-15-100-5-ia-ii.csv", header=FALSE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
names(mt_ia_ii) <- c("min_tokens", "precision", "recall")

# CLEAN min_tokens
mt$min_tokens <- as.integer(gsub("mt-", "", mt$min_tokens))
mt_ia$min_tokens <- gsub("mt-", "", mt_ia$min_tokens)
mt_ia$min_tokens <- as.integer(gsub("-ia", "", mt_ia$min_tokens))
mt_ia_ii$min_tokens <- gsub("mt-", "", mt_ia_ii$min_tokens)
mt_ia_ii$min_tokens <- as.integer(gsub("-ia-ii", "", mt_ia_ii$min_tokens))

# FILTER DATA
mt <- mt[mt$min_tokens<=70,]
mt_ia <- mt_ia[mt_ia$min_tokens<=70,]
mt_ia_ii <- mt_ia_ii[mt_ia_ii$min_tokens<=70,]

# GET MAX AND MIN VALUES
min_precision <- min(min(mt$precision, mt_ia$precision), mt_ia_ii$precision)
min_recall <- min(min(mt$recall, mt_ia$recall), mt_ia_ii$recall)
min_y <- min(min_precision, min_recall)

max_precision <- max(max(mt$precision, mt_ia$precision), mt_ia_ii$precision)
max_recall <- max(max(mt$recall, mt_ia$recall), mt_ia_ii$recall)
max_y <- max(max_precision, max_recall)

# PLOT DATA
pdf("precision-recall.pdf", bg="transparent", width=6, height=3)
par(
  mar = c(4.0, 4.0, 1.1, 0.2)+0.1, # margins (bottom, left, top, right)
  cex.main=1.2,
  cex.lab=1.2,
  cex.axis=1.2
)

# colors
gray_light <- adjustcolor("gray48", alpha.f=1)
gray_dark <- adjustcolor("gray34", alpha.f=1)
blue_light <- adjustcolor("skyblue1", alpha.f=1)
blue_dark <- adjustcolor("steelblue4", alpha.f=1)
red_light <- adjustcolor("salmon", alpha.f=1)
red_dark <- adjustcolor("firebrick3", alpha.f=1)

min_x <- min(mt$min_tokens)
max_x <- max(mt$min_tokens)

# start plot
plot(mt$min_tokens, mt$precision, pch=4, col=gray_light, xlim=c(min_x, max_x), ylim=c(floor(min_y*10)/10, ceiling(1.05*max_y*10)/10), xaxt="n", yaxt="n", xlab="", ylab="", yaxs="i", main="Comparison of CPD configurations")
axis(1, at = seq(min_x, max_x, by = 5))
axis(2, at = seq(floor(min_y*10)/10, ceiling(max_y*10)/10, by = 0.1))
title(xlab="minimum tokens", font.lab=3)
title(ylab="precision & recall", font.lab=3)

points(mt$min_tokens, mt$recall, pch=20, col=gray_dark)
lines(spline(mt$min_tokens, mt$precision, method = c("natural"), n = 100), lty=2, col=gray_dark)
lines(spline(mt$min_tokens, mt$recall, method = c("natural"), n = 100), lty=1, col=gray_dark)

points(mt_ia$min_tokens, mt_ia$precision, pch=4, col=blue_dark)
points(mt_ia$min_tokens, mt_ia$recall, pch=20, col=blue_dark)
lines(spline(mt_ia$min_tokens, mt_ia$precision, method = c("natural"), n = 100), lty=2, col=blue_dark)
lines(spline(mt_ia$min_tokens, mt_ia$recall, method = c("natural"), n = 100), lty=1, col=blue_dark)

points(mt_ia_ii$min_tokens, mt_ia_ii$precision, pch=4, col=red_dark)
points(mt_ia_ii$min_tokens, mt_ia_ii$recall, pch=20, col=red_dark)
lines(spline(mt_ia_ii$min_tokens, mt_ia_ii$precision, method = c("natural"), n = 100), lty=2, col=red_dark)
lines(spline(mt_ia_ii$min_tokens, mt_ia_ii$recall, method = c("natural"), n = 100), lty=1, col=red_dark)

# calcuate values for candidates
precision_25_mt <- mt[mt$min_tokens==25,]$precision  # 0.85
recall_25_mt <- mt[mt$min_tokens==25,]$recall  # 0.53
precision_30_mt <- mt[mt$min_tokens==30,]$precision  # 0.84
recall_30_mt <- mt[mt$min_tokens==30,]$recall  # 0.48
precision_40_mt <- mt[mt$min_tokens==40,]$precision  # 0.94
recall_40_mt <- mt[mt$min_tokens==40,]$recall  # 0.35
precision_50_mt_ia_ii <- mt_ia_ii[mt_ia_ii$min_tokens==50,]$precision  # 0.79
recall_50_mt_ia_ii <- mt_ia_ii[mt_ia_ii$min_tokens==50,]$recall  # 0.42

# draw candidates
segments(25, 0, 25, precision_25_mt, lty=3, col="black")
segments(25, precision_25_mt, 0, precision_25_mt, lty=3, col="black")
segments(25, recall_25_mt, 0, recall_25_mt, lty=3, col="black")

segments(40, 0, 40, precision_40_mt, lty=3, col="black")
segments(40, precision_40_mt, 0, precision_40_mt, lty=3, col="black")
segments(40, recall_40_mt, 0, recall_40_mt, lty=3, col="black")

dev.off() 
