NAME="so-mt40"

#setwd(paste("/Users/sebastian/git/kiefer/stackoverflow-in-github/references-recall/java-high-popularity-3000-unrated/eval/", NAME, sep=""))
setwd(paste("E:/Git/kiefer/stackoverflow-in-github/references-recall/java-high-popularity-3000-unrated/eval/", NAME, sep=""))

# READ DATA
data <- read.csv(file=paste(NAME, "_merged.csv", sep=""), header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
    
# CLEAN DATA
# make relative paths from absolute paths
data$snippet_file <- gsub("/home/sebastian/git/kiefer/snippet-sets/", "", data$snippet_file)
data$occurrence_file <- gsub("/shared/repositories/java-high-popularity-3000-unrated/", "", data$occurrence_file)
nrow(data)
# 18 
sort(table(data$snippet_file), decreasing=TRUE)

unique_snippet_file <- unique.data.frame(data[,c("snippet_file", "occurrence_file")])
# randomly chose one file for each unique combination of file and snippet
unique_snippet_file_pos <- as.integer(apply(unique_snippet_file, 1, function(row) {
    filter <- (data$snippet_file == row[1]) & (data$occurrence_file == row[2])
    snippet_file_pos <- which(filter)
    # if first argument of sample has length 1, the return value is sample(1:x), not x
    if (length(snippet_file_pos)>1) {
        sample(snippet_file_pos, 1)
    } else {
        snippet_file_pos 
    }    
}))

unique_snippet_file_data <- data[unique_snippet_file_pos,]
sort(table(unique_snippet_file_data$snippet_file), decreasing=TRUE)

# extract repo names from paths
library(stringr)
repo_names <- unlist(lapply(unique_snippet_file_data$occurrence_file, function(path) {
    str_extract(path, "^[^/]+")   
}))
unique_snippet_file$repo_name <- repo_names
unique_snippet_file_data$repo_name <- repo_names

sort(table(unique_snippet_file_data$repo_name), decreasing=TRUE)

#draw sample for analyis of false positives
sample_pos <- sample(unique_snippet_file_pos, min(length(unique_snippet_file_pos), 100))
sample_unique_snippet_file_data <- data[sample_pos,]

# WRITE DATA
write.table(unique_snippet_file_data, file=paste(NAME, "_merged_unique.csv", sep=""), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double")
#write.table(sample_unique_snippet_file_data, file=paste(NAME, "_merged_unique_sample.csv", sep=""), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double")

# ANALYSIS
library(sqldf)
match_count <- sqldf('SELECT snippet_file, occurrence_file, COUNT(*) AS match_count FROM data GROUP BY snippet_file, occurrence_file ORDER BY match_count DESC;')
table(match_count$match_count)
#  1  2 
# 14  2 
summary(match_count$match_count)
#  Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 1.000   1.000   1.000   1.125   1.000   2.000  

# WRITE DATA
write.table(match_count, file=paste(NAME, "_match_count.csv", sep=""), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double")

# READ LIST OF FILES WITH REFERENCE
files_with_refs <- read.csv(file="../../files-with-references_all.csv", header=FALSE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE) 
names(files_with_refs) <- c("file")

# REDUCE SNIPPET FILE TO ID
unique_snippet_file$snippet_id <- trimws(gsub("gh/", "", unique_snippet_file$snippet_file)) 
for (i in 1:10) {
    unique_snippet_file$snippet_id <- trimws(gsub(paste("/", i, ".java", sep=""), "", unique_snippet_file$snippet_id)) 
}
unique_snippet_file$snippet_id <- trimws(gsub(".java", "", unique_snippet_file$snippet_id)) 

matches_with_ref <- merge(unique_snippet_file, files_with_refs, by.x="occurrence_file", by.y="file")

# WRITE DATA
write.table(matches_with_ref, file=paste(NAME, "_matches_with_ref.csv", sep=""), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double")

nrow(data)
# 18
nrow(unique_snippet_file)
# 16
nrow(matches_with_ref)
# 2
length(unique(unique_snippet_file$snippet_file))
# 13
length(unique(unique_snippet_file$snippet_id))
# 13
length(unique(unique_snippet_file$occurrence_file))
# 16
length(unique(unique_snippet_file_data$repo_name))
# 15

match_count_repo <- sqldf('SELECT repo_name, COUNT(snippet_file) AS match_count FROM unique_snippet_file_data GROUP BY repo_name ORDER BY match_count DESC;')
table(match_count_repo$match_count)
#  1  2 
# 14  1 

# WRITE DATA FOR GLOBAL COMPARISION
write.table(unique_snippet_file, file=paste(NAME, "_unique_snippet_file.csv", sep=""), sep=",", col.names=TRUE, row.names=FALSE, na="", quote=TRUE, qmethod="double")
